#----------------------------------------------------------------------
# Copyright (c) 2003-2004 free-EOS
# Author:       Emmanuel Saracco <esaracco@noos.fr>, Free-EOS
# 		
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 		
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 		
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#----------------------------------------------------------------------

package eos::FormMagick::Panel::clamav;

use strict;
#use DBI;
#use eos::AccountsDB;
#use eos::util;
use esmith::FormMagick;
#use File::Basename;
use POSIX;
use Exporter;
use Carp;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw(
  form_remove_email
  form_resend_email
  form_set_status_clamd
  form_update
  form_quarantine
  print_update_page
  print_status_button
  print_links
  print_quarantine_page
  print_viewlog_page
  print_viewmail_page
  print_resend_page
  print_remove_page
  print_email
  print_email_infos
  print_log
  print_standard_panel
  goto_previous
  print_alert_page
  form_alert
);

our $configdb = esmith::ConfigDB->open();

sub new 
{
  my $this = esmith::FormMagick->new();
  $this->{calling_package} = shift;
  bless $this;

  $this->init_module ();

  return $this;
}

sub init_module
{
  my $rec = undef;

  # ClamavQuarantinePurgeCronDay
  ($configdb->new_record ('ClamavQuarantinePurgeCronDay'))->set_value ('15')
    if (not defined $configdb->get ('ClamavQuarantinePurgeCronDay'));

  # ClamavQuarantinePurge
  ($configdb->new_record ('ClamavQuarantinePurge'))->set_value ('no')
    if (not defined $configdb->get ('ClamavQuarantinePurge'));

  # ClamavProblemsPurgeCronDay
  ($configdb->new_record ('ClamavProblemsPurgeCronDay'))->set_value ('15')
    if (not defined $configdb->get ('ClamavProblemsPurgeCronDay'));

  # ClamavQuarantinePurge
  ($configdb->new_record ('ClamavQuarantinePurge'))->set_value ('no')
    if (not defined $configdb->get ('ClamavQuarantinePurge'));

  # ClamavProblemsPurge
  ($configdb->new_record ('ClamavProblemsPurge'))->set_value ('no')
    if (not defined $configdb->get ('ClamavProblemsPurge'));

  # FreshclamRefreshDB
  ($configdb->new_record ('FreshclamRefreshDB'))->set_value ('yes')
    if (not defined $configdb->get ('FreshclamRefreshDB'));

  # FreshclamChecks
  ($configdb->new_record ('FreshclamChecks'))->set_value ('24')
    if (not defined $configdb->get ('FreshclamChecks'));

  # AdminNotify
  ($configdb->new_record ('AdminNotify'))->set_value ('yes')
    if (not defined $configdb->get ('AdminNotify'));

  # SenderNotify
  ($configdb->new_record ('SenderNotify'))->set_value ('yes')
    if (not defined $configdb->get ('SenderNotify'));

  # ReceiverNotify
  ($configdb->new_record ('ReceiverNotify'))->set_value ('yes')
    if (not defined $configdb->get ('ReceiverNotify'));
}

sub print_log
{
  my ($this, $type) = @_;
  my $name = $this->get_name_by_type ($type);
  my $file = $this->cgi->param ('name');
  my @content = ();

  $this->exit_if_unsecure ($file);

  open (H, "/var/spool/amavis-ng/$name/$file.log");
  @content = <H>;
  close (H);

  print qq(<textarea cols=80 rows=30>@content</textarea>\n);

  return undef;
}

sub get_log_values
{
  my ($this, $file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;

  if (open (H, $file))
  {
    while (defined ($item = <H>) and ($i <= $#names))
    {
      foreach my $name (@names)
      {
        if ($item =~ /: $name: .*\n$/)
        {
          $i++;
          $item =~ s/^.*: $name:.*: //;
          $header{$name} = $item;
        }
      }
    }
    close (H);
  }

  return %header;
}

sub is_repository_empty
{
  my ($this, $dir) = @_;
  my @files = ();

  opendir (DIR, $dir);
  @files = readdir(DIR);
  closedir (DIR);

  return $#files - 1;
}

sub cut_string
{
  my ($this, $string, $max_len) = @_;

  $string = (substr ($string, 0, $max_len - 5)) . ' [..]'
    if (($max_len > 5) and (length ($string) >= $max_len));

  return $string;
}

sub get_email_header_values
{
  my ($this, $file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;

  if (open (H, $file))
  {
    while (defined ($item = <H>) and ($i <= $#names))
    {
      foreach my $name (@names)
      {
        if ($item =~ /$name: .*\n$/)
        {
          $i++;
          $item =~ s/$name: //;
          $header{$name} = $item;
        }
      }
    }
    close (H);
  }

  return %header;
}

sub print_email
{
  my ($this, $type) = @_;
  my $dir = $this->get_name_by_type ($type);
  my $file = $this->cgi->param ('name');
  my @content = ();

  $this->exit_if_unsecure ($file);

  open (H, "/var/spool/amavis-ng/$dir/$file.msg");
  @content = <H>;
  close (H);

  print qq(<textarea cols=80 rows=30>@content</textarea>\n);

  return undef;
}

sub remove_email
{
  my ($this, $type, $name) = @_;
  my $dir = $this->get_name_by_type ($type);
  my $ret = 1;

  $this->exit_if_unsecure ($name);

  $ret = system ("/sbin/e-smith/signal-event", "clamd-delete", $dir, $name);

  return $ret;
}

sub print_combo_frequencies
{
  my ($this, $selected) = @_;
  my $default = '';

  print qq(<div class="field"><select name="frequency">\n);
  foreach (1..50)
  {
    $default = ($_ == $selected) ? ' SELECTED' : '';
    print qq(<option value="$_"$default>$_</option>\n);
  }
  print qq(</select></div>\n);
}

sub print_combo_all_days
{
  my ($this, $selected) = @_;
  my $default = '';

  print qq(<div class="field"><select name="day">\n);
  foreach (1..31)
  {
    $default = ($_ == $selected) ? ' SELECTED' : '';
    print qq(<option value="$_"$default>$_</option>\n);
  }
  print qq(</select></div>\n);
}

sub print_combo_hours
{
  my ($this, $selected) = @_;
  my $default = '';

  print qq(<div class="field"><select name="hour">\n);
  foreach (0..23)
  {
    $default = ($_ == $selected) ? ' SELECTED' : '';
    print qq(<option value="$_"$default>$_</option>\n);
  }
  print qq(</select></div>\n);
}

sub print_combo_days
{
  my ($this, $selected) = @_;
  my $default = '';
  my $msg_everyday = $this->localise ('EVERYDAY');
  my @days = (
    $this->localise ('SUNDAY'),
    $this->localise ('MONDAY'),
    $this->localise ('TUESDAY'),
    $this->localise ('WEDNESDAY'),
    $this->localise ('THURSDAY'),
    $this->localise ('FRIDAY'),
    $this->localise ('SATURDAY')
  );

  print qq(<div class="field"><select name="day">\n);
  $default = ($selected == 7) ? ' SELECTED' : '';
  print qq(<option value="7"$default>$msg_everyday</option>\n);
  for (my $i = 0; $i < 7; $i++)
  {
    my $value = $days[$i];
    my $default = ($i == $selected) ? ' SELECTED' : '';

    print qq(<option value="$i"$default>$value</option>\n);
  }
  
  print qq(</select></div>\n);
}

sub print_update_page
{
  my $this = shift;
  my $default_freq = ($configdb->get ('FreshclamChecks'))->value ();
  my $refresh_rec = $configdb->get ('FreshclamRefreshDB');
  my $proxy_host = ($configdb->get ('SquidParent'))->value ();
  my $proxy_port = ($configdb->get ('SquidParentPort'))->value ();
  my $msg_title1 = $this->localise ('UPDATE_TITLE_MANUAL');
  my $msg_title2 = $this->localise ('UPDATE_TITLE_AUTO');
  my $msg_desc1 = $this->localise ('UPDATE_PAGE_DESCRIPTION_GENERAL');
  my $msg_desc2= $this->localise ('UPDATE_PAGE_DESCRIPTION_MANUAL');
  my $msg_desc3= $this->localise ('UPDATE_PAGE_DESCRIPTION_AUTO');
  my $msg_desc4= $this->localise ('UPDATE_PAGE_PROXY_DESCRIPTION');
  my $msg_freq= $this->localise ('FREQUENCY');
  my $msg_norefresh = $this->localise ('NEVER_REFRESH');
  my $msg_proxy_host = $this->localise ('PROXY_HOST');
  my $msg_proxy_port = $this->localise ('PROXY_PORT');
  my $default_refresh = ($refresh_rec->value () eq 'no') ? ' checked' : '';

  print $this->show_logfile();

  print qq(<p class="pagedescription">$msg_desc1</p>);

  if ($proxy_host and $proxy_port)
  {
    print qq(<p class="pagedescription">$msg_desc4</p>);
    print qq(<p><table border="0">);
    print qq(<tr><td class="label"><i>$msg_proxy_host</i></td>
      <td class="field">);
    print qq($proxy_host</td>);
    print qq(</tr><tr><td class="label"><i>$msg_proxy_port</i></td>
      <td class="field">);
    print qq($proxy_port</td>);
    print qq(</tr></table></p>);
  }

  print qq(<h2>$msg_title1</h2>\n);
  print qq(<p class="pagedescription">$msg_desc2</p>\n);
  print $this->print_standard_panel ($this->localise('UPDATE_NOW'));

  print qq(<h2>$msg_title2</h2>\n);
  print qq(<p class="pagedescription">$msg_desc3</p>\n);
  print qq(<form>);
  print qq(<p><table border=1>\n); 
  print qq(<tr><td><b>$msg_freq</b></td></tr>\n);
  print qq(<tr>);
  print qq(<td align="center">); 
  $this->print_combo_frequencies ($default_freq); print qq(</td>);
  print qq(</tr>);
  print qq(</table></p>\n);

  print qq(<table border=0>\n);
  print qq(<tr><td><div class="field"><input type="checkbox" name="norefresh");
  print qq( value="on"$default_refresh></div></td>\n);
  print qq(<td><div class="label">$msg_norefresh</div></td></tr>\n);
  print qq(</table>\n);
 
  print $this->print_standard_panel (
    $this->localise('SAVE'), $this->localise('RETURN'));
  print qq(</form>);
  return undef;
}

sub print_alert_page
{
  my $this = shift;
  my $admin_notify = $configdb->get ('AdminNotify')->value () || 'no';
  my $sender_notify = $configdb->get ('SenderNotify')->value() || 'no';
  my $receiver_notify = $configdb->get ('ReceiverNotify')->value () || 'no';
  my $msg_title1 = $this->localise ('ALERT_TITLE_DEST');
  my $msg_desc1 = $this->localise ('ALERT_PAGE_DESCRIPTION_GENERAL');
  my $msg_desc2 = $this->localise ('ALERT_NOTIFY_ADMIN_DESCRIPTION');
  my $msg_desc3 = $this->localise ('ALERT_NOTIFY_SENDER_DESCRIPTION');
  my $msg_desc4 = $this->localise ('ALERT_NOTIFY_RECEIVER_DESCRIPTION');
  my $default_admnot = ($admin_notify eq 'yes') ? 'checked' : '';
  my $default_sendnot = ($sender_notify eq 'yes') ? 'checked' : '';
  my $default_receivnot = ($receiver_notify eq 'yes') ? 'checked' : '';


  print qq(<p class="pagedescription">$msg_desc1</p>);
  print qq(<h2>$msg_title1</h2>\n);
  print qq(<form><table border=0>\n);
  print qq(<tr><td><div class="label">$msg_desc2</div></td>\n);
  print qq(<td><div class="field"><input type="checkbox" name="admnot");
  print qq( value="yes"$default_admnot></div></td></tr>\n);
  print qq(<tr><td><div class="label">$msg_desc3</div></td>\n);
  print qq(<td><div class="field"><input type="checkbox" name="sendnot");
  print qq( value="yes"$default_sendnot></div></td></tr>\n);
  print qq(<tr><td><div class="label">$msg_desc4</div></td>\n);
  print qq(<td><div class="field"><input type="checkbox" name="receivnot");
  print qq( value="yes"$default_receivnot></div></td></tr>\n);
  print qq(</table><p>\n);
  print $this->print_standard_panel (
    $this->localise('SAVE'), $this->localise('RETURN'));
  print qq(</form><p>\n);
 
  return undef;

}

sub form_remove_email
{
  my $this = shift;
  my $type = $this->cgi->param ('type');
  my $action = $this->cgi->param ('Next');
  my $name = $this->cgi->param ('name');

  if ($action eq $this->localise ('DELETE'))
  {
    ($this->remove_email ($type, $name) != 0) ?
      $this->set_status_message ("MSG_ERROR_STATUS_REMOVE") :
      $this->set_status_message ("MSG_SUCCESS_STATUS_REMOVE")
  }

  $this->wherenext ('quarantine_page');
}

sub resend_email
{
  my ($this, $type, $name) = @_;
  my $dir = $this->get_name_by_type ($type);
  my $ret = 0;

  $this->exit_if_unsecure ($name);

  $ret = system ("/sbin/e-smith/signal-event", "clamd-resend", $dir, $name);

  return $ret;
}

sub form_resend_email
{
  my $this = shift;
  my $type = $this->cgi->param ('type');
  my $action = $this->cgi->param ('Next');
  my $name = $this->cgi->param ('name');

  if ($action eq $this->localise ('RESEND'))
  {
    ($this->resend_email ($type, $name) == 0) ?
      $this->set_status_message ("MSG_ERROR_STATUS_RESEND") :
      $this->set_status_message ("MSG_SUCCESS_STATUS_RESEND");

    $this->remove_email ($type, $name);
  }

  $this->wherenext ('quarantine_page');
}

sub button_off_status_on
{
  my $this = shift;
 
  $this->turn_off_buttons ();
  $this->print_status_message ();

  return undef;
}

sub goto_previous
{
  my ($this, $previous) = @_;
  my $next = $this->cgi->param ('Next');

  $this->wherenext ("$previous") if ("$next" eq $this->localise ('RETURN'));
}

sub form_update
{
  my $this = shift;
  my $next = $this->cgi->param ('Next');
  my $freq = $this->cgi->param ('frequency');
  my $norefresh = $this->cgi->param ('norefresh');
  my $wherenext = '';
  my $file = '';

  if ($next eq $this->localise ('UPDATE_NOW'))
  {
    $this->update ();
 
    $this->set_status_message ("MSG_SUCCESS_UPDATE_MANUAL");
    $wherenext = 'update_page';
  }
  elsif ($next eq $this->localise ('SAVE'))
  {
    ($configdb->get ('FreshclamChecks'))->set_value ($freq);
    if ($norefresh eq 'on')
    {
      ($configdb->get ('FreshclamRefreshDB'))->set_value ('no');
      $this->set_status ('freshclam', 'disabled');
    }
    else
    {
      ($configdb->get ('FreshclamRefreshDB'))->set_value ('yes');
      $this->set_status ('freshclam', 'enabled');
    }

    system ("/sbin/e-smith/expand-template /etc/freshclam.conf");

    $this->set_status_message ("MSG_SUCCESS_UPDATE_AUTO");
    $wherenext = 'index_page';
  }
  else
  {
    $wherenext = 'index_page';
  }
    
  $this->wherenext ($wherenext);
}

sub form_alert
{
  my $this = shift;
  my $next = $this->cgi->param ('Next');
  my $admnot = $this->cgi->param ('admnot') || 'no';
  my $sendnot = $this->cgi->param ('sendnot') || 'no';
  my $receivnot = $this->cgi->param ('receivnot') || 'no';
  my $wherenext = '';
  my $file = '';

  if ($next eq $this->localise ('SAVE'))
  {
    ($configdb->get ('AdminNotify'))->set_value ($admnot);
    ($configdb->get ('SenderNotify'))->set_value ($sendnot);
    ($configdb->get ('ReceiverNotify'))->set_value ($receivnot);

    system ("/sbin/e-smith/expand-template /etc/amavis-ng/amavis.conf");

    $this->set_status_message ("MSG_SUCCESS_UPDATE_NOTIFY");
    $wherenext = 'index_page';
  }
  else
  {
    $wherenext = 'index_page';
  }
    
  $this->wherenext ($wherenext);
}

sub rescan_email
{
  my ($this, $type, $name) = @_;
  my $dir = $this->get_name_by_type ($type);
  my @array = ();
  my $ret = '';
  my $code = 0;

  $this->exit_if_unsecure ($name);

  # untaint the variable
  $name =~ /^(\w+-\w+)$/; $name = $1;

  open (H, "/bin/cat /var/spool/amavis-ng/$dir/$name.msg | " .
    "/usr/bin/clamscan --no-summary --mbox - 2>&1 |");
  while (<H>) {$ret .= "$_<br>";}
  close (H);

  # if virus found
  if ($ret =~ /^.*:\s+(.*)\s+FOUND/)
    {$code = 1; $ret = sprintf ($this->localise ('MSG_ERROR_VIRUS_FOUND'), $1);}
  # else ok
  else
    {$code = 0; $ret = $this->localise ('MSG_SUCCESS_VIRUS_NOT_FOUND');};

  return ($code, $ret);
}

sub check_and_resend_all
{
  my ($this, $type) = @_;
  my $dir = $this->get_name_by_type ($type);
  my $count_bad = 0;
  my $count_ok = 0;

  opendir (DIR, "/var/spool/amavis-ng/$dir/");
  while (my $file = readdir (DIR))
  {
    next if not ($file =~ /^(.*)\.msg$/);

    my $name = $1;
    my ($code, $ret) = $this->rescan_email ($type, $name);

    # if scan ok, resend email
    if ($code == 0)
    {
      $count_ok++;
      $this->resend_email ($type, $name); 
    }
    else
      {$count_bad++;}
  }
  closedir (DIR);

  return ($count_ok, $count_bad);
}

sub form_quarantine
{
  my $this = shift;
  my $next = $this->cgi->param ('Next');
  my $day = $this->cgi->param ('day');
  my $name = $this->cgi->param ('name') || '';
  my $type = $this->cgi->param ('type') || 1;
  my $nopurge = $this->cgi->param ('nopurge');
  my $dir = $this->get_name_by_type ($type);
  my $wherenext = '';
  my $file = '';
  my $ret = '';

  return if (!$next);

  # check email (problems)
  if ($next eq 'check')
  {
    ($_, $ret) = $this->rescan_email ($type, $name);
    $this->set_status_message ($ret);

    $wherenext = 'quarantine_page';
  }
  # resend all emails (problems)
  elsif ($next eq $this->localise ('PROBLEMS_RESEND_CLICK_HERE'))
  {
    my ($count_ok, $count_bad) = $this->check_and_resend_all ($type);

    $this->set_status_message (
      sprintf ($this->localise ('PROBLEMS_RESEND_REPORT'), 
        $count_bad, $count_ok)
    );

    $wherenext = 'quarantine_page';
  }
  # purge now
  elsif ($next eq $this->localise ('MANUAL_PURGE'))
  {
    opendir(DIR, "/var/spool/amavis-ng/$dir") or die "can't opendir $dir: $!";
    while (defined($file = readdir(DIR))) {
    next if $file =~ /^\.\.?$/;     # skip . and ..
    $file =~ /^([-\@\w.]+)$/; $file = $1;
    $dir =~ /^([-\@\w.]+)$/; $dir = $1;
    unlink "/var/spool/amavis-ng/$dir/$file" or die "Can't delete /var/spool/amavis-ng/$dir/$file: $!\n"
	}
    closedir(DIR);	
    $wherenext = 'index_page';
  }
  # save purge settings
  elsif ($next eq $this->localise ('SAVE'))
  {
    my $var = $dir;

    substr ($var, 0, 1) = uc (substr ($var, 0, 1));

    ($configdb->get ("Clamav${var}PurgeCronDay"))->set_value ($day);
    ($configdb->get ("Clamav${var}Purge"))->set_value (
      ($nopurge eq 'on') ? 'no' : 'yes'
    );

    system ("/sbin/e-smith/expand-template /etc/cron.d/clamav_quarantine.cron");

    $this->set_status_message ("MSG_SUCCESS_UPDATE_PURGE");
    $wherenext = 'quarantine_page';
  }
  # return to index page
  else
  {
    $wherenext = 'index_page';
  }
    
   $this->wherenext ($wherenext);
}

sub show_logfile
{
  my $this = shift;

  print qq(<pre>\n);
  if (open (H, "/tmp/.freshclam.$$.log"))
  {
    print $_ while (<H>);
    close (H);
  }
  print qq(</pre>\n);
  system ("/bin/rm -f /tmp/.freshclam.$$.log");

  return undef
}

sub form_set_status_clamd
{
  my $this = shift;
  my $status = $this->cgi->param ('status');
  my $msg = '';

  if ($status eq $this->localise ('ACTIVATE'))
  {
    $this->set_status ('clamd', 'enabled');
    $msg = "MSG_SUCCESS_STATUS_ENABLED";
  }
  else
  {
    $this->set_status ('clamd', 'disabled');
    $msg = "MSG_SUCCESS_STATUS_DISABLED";
  }

  $this->set_status_message ($msg);
  $this->wherenext ('index_page');
}

sub exit_if_unsecure
{
  my ($this, $str) = @_;

  exit (1)
     if ($str =~ /\// or $str =~ /\.\./ or $str =~ /;/);
}

sub print_email_infos
{
  my ($this, $type) = @_;
  my $dir = $this->get_name_by_type ($type);
  my $name = $this->cgi->param ('name');
  my $msg_subject = $this->localise ("SUBJECT");
  my $msg_from = $this->localise ("FROM");
  my $msg_to = $this->localise ("TO");
  my %header = $this->get_email_header_values (
      "/var/spool/amavis-ng/$dir/$name.msg",
      qw(Subject X-Quarantined-From X-Quarantined-To)
    );
  my $subject = $header{'Subject'};
  my $from = $header{'X-Quarantined-From'};
  my $to = $header{'X-Quarantined-To'};

  $this->exit_if_unsecure ($name);

  print qq(<input type="hidden" name="name" value="$name">\n);
  print qq(<table border=0>\n);
  print qq(<tr><td class="label">$msg_subject: </td>);
  print qq(<td class="field">$subject</td><tr>\n);
  print qq(<tr><td class="label">$msg_from: </td>);
  print qq(<td class="field">$from</td><tr>\n);
  print qq(<tr><td class="label">$msg_to: </td>);
  print qq(<td class="field">$to</td><tr>\n);
  print qq(</table><p>\n);

  return undef;
}

sub print_quarantine_page
{
  my $this = shift;
  my $type = $this->cgi->param ('type');
  my $name = $this->get_name_by_type ($type);
  my $msg_title = '';
  my $msg_desc = '';

  $name = uc ($name);
  $msg_title = $this->localise ("${name}_PAGE_TITLE");
  $msg_desc = $this->localise ("${name}_PAGE_DESCRIPTION");
  $name = lc ($name);

  print qq(<input type="hidden" name="type" value="$type">);
  print qq(<h2>$msg_title</h2>);
  print qq(<p class="pagedescription">$msg_desc</p>);

  $this->print_quarantine_table ($type);
  # if "problems"
  if ($type == 2)
  {
    my $click_here = $this->localise ('PROBLEMS_RESEND_CLICK_HERE');

    print $this->localise ('PROBLEMS_RESEND_ALL');
    print qq(<input type="submit" name="Next" value="$click_here">);
  }
  $this->print_quarantine_purge_section ($type);
}

sub print_viewlog_page
{
  my $this = shift;
  my $type = $this->cgi->param ('type');
  my $name = $this->get_name_by_type ($type);
  my $msg_title = '';
  my $msg_desc = '';

  $name = uc ($name);
  $msg_title = $this->localise ("${name}_VIEWLOG_PAGE_TITLE");
  $msg_desc = $this->localise ("${name}_VIEWLOG_PAGE_DESCRIPTION");
  $name = lc ($name);

  print qq(<h2>$msg_title</h2>);
  print qq(<p class="pagedescription">$msg_desc</p>);

  $this->print_log ($type);
}

sub print_remove_page
{
  my $this = shift;
  my $type = $this->cgi->param ('type');
  my $name = $this->get_name_by_type ($type);
  my $msg_title = '';
  my $msg_desc = '';

  $name = uc ($name);
  $msg_title = $this->localise ("${name}_REMOVE_PAGE_TITLE");
  $msg_desc = $this->localise ("${name}_REMOVE_PAGE_DESCRIPTION");
  $name = lc ($name);

  print qq(<h2>$msg_title</h2>);
  print qq(<p class="pagedescription">$msg_desc</p>);

  print qq(<input type="hidden" name="type" value="$type">);

  $this->print_email_infos ($type);
}

sub print_resend_page
{
  my $this = shift;
  my $type = $this->cgi->param ('type');
  my $name = $this->get_name_by_type ($type);
  my $msg_title = '';
  my $msg_desc = '';

  $name = uc ($name);
  $msg_title = $this->localise ("${name}_RESEND_PAGE_TITLE");
  $msg_desc = $this->localise ("${name}_RESEND_PAGE_DESCRIPTION");
  $name = lc ($name);

  print qq(<h2>$msg_title</h2>);
  print qq(<p class="pagedescription">$msg_desc</p>);

  print qq(<input type="hidden" name="type" value="$type">);

  $this->print_email_infos ($type);
}

sub print_viewmail_page
{
  my $this = shift;
  my $type = $this->cgi->param ('type');
  my $name = $this->get_name_by_type ($type);
  my $msg_title = '';
  my $msg_desc = '';

  $name = uc ($name);
  $msg_title = $this->localise ("${name}_VIEWMAIL_PAGE_TITLE");
  $msg_desc = $this->localise ("${name}_VIEWMAIL_PAGE_DESCRIPTION");
  $name = lc ($name);

  print qq(<h2>$msg_title</h2>);
  print qq(<p class="pagedescription">$msg_desc</p>);

  print qq(<input type="hidden" name="type" value="$type">);

  $this->print_email ($type);
}

sub print_quarantine_purge_section
{
  my ($this, $type) = @_;
  my $name = $this->get_name_by_type ($type);
  my $default_purge = '';
  my $default_day = '';
  my $purge_rec = '';
  my $msg_title = '';
  my $msg_description = '';
  my $var = $name;
  my $msg_day = $this->localise ("DAY");
  my $msg_nopurge = $this->localise ("NEVER_PURGE");
  my $msg_manual_purge = $this->localise ("MANUAL_PURGE");
  my $msg_delete_emails = $this->localise ("DELETE_EMAILS");
  my $msg_delete_emails_days = $this->localise ("DELETE_EMAILS_DAYS");

  substr ($var, 0, 1) = uc (substr ($var, 0, 1)); 
  $default_day = ($configdb->get ("Clamav${var}PurgeCronDay"))->value ();
  $purge_rec = $configdb->get ("Clamav${var}Purge");
  $default_purge = ($purge_rec->value () eq 'no') ? ' checked' : '';

  $name = uc ($name);
  $msg_title = $this->localise ("${name}_PURGE_TITLE");
  $msg_description = $this->localise ("${name}_PURGE_DESCRIPTION");
  $name = lc ($name);

  print qq(<h2>$msg_title</h2>);
  print qq(<p class="description">$msg_description</p>);

  print qq(<b>Manuel</b>);
  print qq(<p><input type="submit" name="Next" value="$msg_manual_purge"></p>);

  print qq(<b>Automatique</b>);

  print qq(<table border=0 cellpadding=2 cellspacing=2>\n); 
  print qq(<tr>);
  print qq(<td>$msg_delete_emails</td><td>); 
  $this->print_combo_all_days ($default_day); 
  print qq(</td><td>$msg_delete_emails_days</td>);
  print qq(</tr>);
  print qq(</table>\n);

  print qq(<table border=0>\n);
  print qq(<tr><td><div class="field"><input type="checkbox" name="nopurge");
  print qq( value="on"$default_purge></div></td>\n);
  print qq(<td><div class="label">$msg_nopurge</div></td></tr>\n);
  print qq(</table>\n);

  print $this->print_standard_panel (
    $this->localise('SAVE'), $this->localise('RETURN'));

  return undef;
}

sub get_name_by_type
{
  my ($this, $type) = @_;
  my $ret = '';

  if ($type == 1) {$ret = 'quarantine';}
    elsif ($type == 2) {$ret = 'problems';}

  return $ret;
}

sub print_quarantine_table
{
  my ($this, $type) = @_;
  my $name = $this->get_name_by_type ($type);
  my @files = ();
  my $i = 0;
  my $STRING_MAX_LEN = 30;
  my $msg_subject = $this->localise ("SUBJECT");
  my $msg_viewlog = $this->localise ("VIEWLOG");
  my $msg_action = $this->localise ("ACTION");
  my $msg_resend = $this->localise ("RESEND");
  my $msg_delete = $this->localise ("DELETE");
  my $msg_check = $this->localise ("CHECK");
  my $msg_from = $this->localise ("FROM");
  my $msg_to = $this->localise ("TO");
  my $msg_virus = $this->localise ("VIRUS");

  opendir (DIR, "/var/spool/amavis-ng/$name");
  @files = readdir (DIR);  
  closedir (DIR);

  print qq(<table border=1>\n);
  print qq(<tr><td><b>$msg_subject</b></td><td><b>$msg_virus</b></td>);
  print qq(<td><b>$msg_from</b></td><td><b>$msg_to</b></td>);
  print qq(<td><b>$msg_action</b></td></tr>\n);

  for ($i = 0; $i < scalar (@files); $i += 2)
  {
    if ($files[$i] ne '.')
    {
      my $log = $files[$i];
      my $msg = $files[$i + 1];
      my $base = $log; $base =~ s/\..*$//g;
      my %header = ();
      %header = $this->get_email_header_values (
          "/var/spool/amavis-ng/$name/$base.msg",
          qw(Subject X-Quarantined-From X-Quarantined-To)
        );
      my $subject = $header{'Subject'};
      my $from = $header{'X-Quarantined-From'};
      my $to = $header{'X-Quarantined-To'};
      %header = $this->get_log_values (
          "/var/spool/amavis-ng/$name/$base.log",
          qw(AMAVIS::AV::CLAMD)
        );
      my $virus = $header{'AMAVIS::AV::CLAMD'} || '';

      $subject = $this->cut_string ($subject, $STRING_MAX_LEN);
      $from = $this->cut_string ($from, $STRING_MAX_LEN);
      $to = $this->cut_string ($to, $STRING_MAX_LEN);
      $virus = $this->cut_string ($virus, $STRING_MAX_LEN);
      $virus = '&nbsp;' if ($virus eq '');
      $from = '&nbsp;' if ($from eq '' or not ($from =~ /[a-zA-Z]/));

      print qq(<tr><td valign="top">);
      print qq(<a href="/server-manager/cgi-bin/clamav?type=$type&page=1);
      print qq(&name=$base&wherenext=quarantine_viewmail_page">$subject</a>);
      print qq(</td><td valign="top">);
      print qq(<b>$virus</b></td><td valign="top">$from</td>);
      print qq(<td valign="top">$to</td>);
      print qq(<td valign="top" nowrap>);
      print qq(<a href="/server-manager/cgi-bin/clamav?type=$type&page=1);
      print qq(&name=$base&wherenext=quarantine_viewlog_page" 
        title="$msg_viewlog">);
      print qq(V</a> / );
      print qq(<a href="/server-manager/cgi-bin/clamav?type=$type&page=1);
      print qq(&name=$base&wherenext=quarantine_resend_page" 
        title="$msg_resend">);
      print qq(R</a> / );

      if ($type == 2)
      {
        print qq(<a href="/server-manager/cgi-bin/clamav?type=$type&page=1);
        print qq(&Next=check&name=$base&wherenext=quarantine_page" 
          title="$msg_check">);
        print qq(C</a> / );
      }

      print qq(<a href="/server-manager/cgi-bin/clamav?type=$type&page=1);
      print qq(&name=$base&wherenext=quarantine_remove_page" 
        title="$msg_delete">);
      print qq(S</a></td></tr>\n);

    }
    else
    {
      $i++;
    }
  }

  print qq(</table>\n<p>\n);

  return undef;
}

sub update
{
  system ("/usr/bin/freshclam --quiet -l /tmp/.freshclam.$$.log");
}

sub set_status
{
  my ($this, $name, $status) = @_;
  my $rec = $configdb->get ($name);

  $rec->set_prop ('status', $status);

  system ("/sbin/e-smith/signal-event", "$name-modify")
}

sub get_status
{
  my ($this, $name) = @_;
  my $rec = $configdb->get ($name);

  return $rec->prop ('status');
}

sub print_standard_panel
{
  my $this = shift;
  my $label = '';

  print qq(<p>);
  if ($_[0])
  {
    $label = $this->localise ($_[0]);
    print qq(<input type="submit" name="Next" value="$label"> );
  } 
  if ($_[1])
  {
    $label = $this->localise ($_[1]);
    print qq(<input type="submit" name="Next" value="$label"> );
  } 
  if ($_[2])
  {
    $label = $this->localise ($_[2]);
    print qq(<input type="submit" name="Next" value="$label"> );
  } 

  return undef;
}

sub print_status_button
{
  my $this = shift;
  my $label = '';

  $label = ($this->get_status ('clamd') eq 'enabled') ? 
    $this->localise ('DEACTIVATE') :
    $this->localise ('ACTIVATE');

  print qq (<input type="submit" name="status" value="$label">);

  return undef;
}

sub get_last_db_update
{
  my $this = shift;
  my $main = '';
  my $daily = '';
  my $path = '/var/lib/clamav';

  $main = strftime('%Y-%m-%d %H:%M', 
    localtime((stat("$path/main.cvd"))[9]))
      if (-e "$path/main.cvd");

  $daily = strftime('%Y-%m-%d %H:%M', 
    localtime((stat("$path/daily.inc/daily.info"))[9]))
      if (-e "$path/daily.inc/daily.info");

  return ($main, $daily);
}

sub print_links
{
  my $this = shift;
  my $msg_quarantine = $this->localise("LINK_QUARANTINE_PAGE");
  my $msg_problems = $this->localise("LINK_PROBLEMS_PAGE");
  my $msg_quarantine_empty = $this->localise("LINK_QUARANTINE_EMPTY_PAGE");
  my $msg_problems_empty = $this->localise("LINK_PROBLEMS_EMPTY_PAGE");
  my $msg_update = $this->localise("LINK_UPDATE_PAGE");
  my $msg_alert = $this->localise("LINK_ALERT_PAGE");
  my $msg_links_desc = $this->localise("LINKS_DESCRIPTION");
  my ($main, $daily) = $this->get_last_db_update ();
  my $msg_db_update = '';

  $msg_db_update = ($main eq '') ?
    $this->localise ('NOT_YET_UPDATED') :
    $this->localise ('MAIN_UPDATE_DATE') . ': ' . $main .', ' .
    $this->localise ('DAILY_UPDATE_DATE') . ': ' . $daily;

  if ($this->get_status ('clamd') eq 'enabled')
  {
    print qq(<p><div class="fielddescription">$msg_links_desc</div></p>);
    print qq(<ul>\n);
    # quarantine
    if ($this->is_repository_empty ('/var/spool/amavis-ng/quarantine') <= 0)
    {
      print "<li>$msg_quarantine ($msg_quarantine_empty)</li>";
    }
    else
    {
      print 
        "<li><a href=\"/server-manager/cgi-bin/clamav?type=1&page=1" .
        "&wherenext=quarantine_page\">$msg_quarantine</a></li>";
    }
    # problems
    if ($this->is_repository_empty ('/var/spool/amavis-ng/problems') <= 0)
    {
      print "<li>$msg_problems ($msg_problems_empty)</li>";
    }
    else
    {
      print 
        "<li><a href=\"/server-manager/cgi-bin/clamav?type=2&page=1" .
        "&wherenext=quarantine_page\">$msg_problems</a></li>";
    }

    print qq(<li><a href="/server-manager/cgi-bin/clamav?page=1);
    print qq(&wherenext=update_page">$msg_update</a> ($msg_db_update)</li>);
    print qq(<li><a href="/server-manager/cgi-bin/clamav?page=1);
    print qq(&wherenext=alert_page">$msg_alert</a> </li>);
    print qq(</ul>\n);
  }

  return undef;
}
1;
